@@ -166,6 +166,7 @@ set minor = `echo $versionstring | cut -d. -f2`
 
 # Sigh. . .  versioning doesn't follow any fixed standard
 set minortest = `echo $minor | cut -d+ -f2`
+set minor = `echo $minor | cut -d+ -f1`
 if ( ${minortest} == "" ) then
 
    set revisionstring = `echo $versionstring | cut -d. -f3`
@@ -176,7 +177,6 @@ if ( ${minortest} == "" ) then
 
 else
    set revision = 0
-   set minor = `echo $minor | cut -d+ -f1`
    set subrevision = ${minortest}
 
 endif
@@ -258,7 +258,7 @@ EOF
 if ( ${?abc_script} ) then
    if ( ${abc_script} != "" ) then
       cat >> ${rootname}.ys << EOF
-abc -liberty ${libertypath} -script ${abc_script}
+abc -exe ${bindir}/yosys-abc -liberty ${libertypath} -script ${abc_script}
 flatten
 
 EOF
@@ -266,7 +266,7 @@ EOF
       echo "Warning: no abc script ${abc_script}, using default, no script" \
 		|& tee -a ${synthlog}
       cat >> ${rootname}.ys << EOF
-abc -liberty ${libertypath}
+abc -exe ${bindir}/yosys-abc -liberty ${libertypath}
 flatten
 
 EOF
@@ -274,7 +274,7 @@ EOF
 else
    cat >> ${rootname}.ys << EOF
 # Map combinatorial cells, standard script
-abc -liberty ${libertypath} -script +strash;scorr;ifraig;retime,{D};strash;dch,-f;map,-M,1,{D}
+abc -exe ${bindir}/yosys-abc -liberty ${libertypath} -script +strash;scorr;ifraig;retime,{D};strash;dch,-f;map,-M,1,{D}
 flatten
 
 EOF
@@ -433,60 +433,70 @@ cat ${final_blif} | sed \
 # Switch to synthdir for processing of the BDNET netlist
 cd ${synthdir}
 
+#---------------------------------------------------------------------
+# If "nofanout" is set, then don't run blifFanout.
+#---------------------------------------------------------------------
+
+if ($?nofanout) then
+   set nchanged=0
+else
+
 #---------------------------------------------------------------------
 # Make a copy of the original blif file, as this will be overwritten
 # by the fanout handling process
 #---------------------------------------------------------------------
 
-cp ${rootname}.blif ${rootname}_bak.blif
+   cp ${rootname}.blif ${rootname}_bak.blif
 
 #---------------------------------------------------------------------
 # Check all gates for fanout load, and adjust gate strengths as
 # necessary.  Iterate this step until all gates satisfy drive
 # requirements.
 #
-# Use option "-o value" to force a value for the (maximum expected)
-# output load, in fF.  Currently, there is no way to do this from the
-# command line (default is 18fF). . .
+# Use option "-c value" in fanout_options to force a value for the
+# (maximum expected) output load, in fF (default is 30fF)
+# Use option "-l value" in fanout_options to force a value for the
+# maximum latency, in ps (default is 1000ps)
 #---------------------------------------------------------------------
 
-rm -f ${rootname}_nofanout
-touch ${rootname}_nofanout
-if ($?gndnet) then
-   echo $gndnet >> ${rootname}_nofanout
-endif
-if ($?vddnet) then
-   echo $vddnet >> ${rootname}_nofanout
-endif
+   rm -f ${rootname}_nofanout
+   touch ${rootname}_nofanout
+   if ($?gndnet) then
+      echo $gndnet >> ${rootname}_nofanout
+   endif
+   if ($?vddnet) then
+      echo $vddnet >> ${rootname}_nofanout
+   endif
 
-if (! $?fanout_options) then
-   set fanout_options="-l 75 -c 25"
-endif
+   if (! $?fanout_options) then
+      set fanout_options=""
+   endif
 
-echo "Running blifFanout (iterative)" |& tee -a ${synthlog}
-echo "" >> ${synthlog}
-if (-f ${libertypath} && -f ${bindir}/blifFanout ) then
-   set nchanged=1000
-   while ($nchanged > 0)
-      mv ${rootname}.blif tmp.blif
-      if ("x${separator}" == "x") then
-	 set sepoption=""
-      else
-	 set sepoption="-s ${separator}"
-      endif
-      if ("x${bufcell}" == "x") then
-	 set bufoption=""
-      else
-	 set bufoption="-b ${bufcell} -i ${bufpin_in} -o ${bufpin_out}"
-      endif
-      ${bindir}/blifFanout ${fanout_options} -f ${rootname}_nofanout \
+   echo "Running blifFanout (iterative)" |& tee -a ${synthlog}
+   echo "" >> ${synthlog}
+   if (-f ${libertypath} && -f ${bindir}/blifFanout ) then
+      set nchanged=1000
+      while ($nchanged > 0)
+         mv ${rootname}.blif tmp.blif
+         if ("x${separator}" == "x") then
+	    set sepoption=""
+         else
+	    set sepoption="-s ${separator}"
+         endif
+         if ("x${bufcell}" == "x") then
+	    set bufoption=""
+         else
+	    set bufoption="-b ${bufcell} -i ${bufpin_in} -o ${bufpin_out}"
+         endif
+         ${bindir}/blifFanout ${fanout_options} -I ${rootname}_nofanout \
 		-p ${libertypath} ${sepoption} ${bufoption} \
 		tmp.blif ${rootname}.blif >>& ${synthlog}
-      set nchanged=$status
-      echo "gates resized: $nchanged" |& tee -a ${synthlog}
-   end
-else
-   set nchanged=0
+         set nchanged=$status
+         echo "gates resized: $nchanged" |& tee -a ${synthlog}
+      end
+   else
+      set nchanged=0
+   endif
 endif
 
 #---------------------------------------------------------------------
